Re: [PATCH] fbcon: don't suspend/resume when vc is graphics mode
From: Helge Deller
Date: Tue May 19 2026 - 04:11:03 EST
Hello Lu,
On 4/30/26 08:01, yaolu@xxxxxxxxxx wrote:
From: Lu Yao <yaolu@xxxxxxxxxx>
Don't need to do suspend/resume for fbcon in graphic mode.
Doing this may cause error, eg:
At the beginning, starting the Xorg with single screen and then an
external screen was plugged in. After logging out in Xorg, fbdev
info may using screen which is connected later on for info always
using first connected connector in list in func 'drm_setup_crtcs_fb'.
Then, S3 executed, fbcon found that the information did not match
and do atomic to switch fb. However, Xorg will not re-bind the crtc
fb but continues doing ioctl. At this time, the fb is incorrect.
Do you still have the possibility to test this issue?
Signed-off-by: Lu Yao <yaolu@xxxxxxxxxx>
---
drivers/video/fbdev/core/fbcon.c | 9 ++++++---
1 file changed, 6 insertions(+), 3 deletions(-)
diff --git a/drivers/video/fbdev/core/fbcon.c b/drivers/video/fbdev/core/fbcon.c
index b0e3e765360d..450e690d0bd2 100644
--- a/drivers/video/fbdev/core/fbcon.c
+++ b/drivers/video/fbdev/core/fbcon.c
@@ -2602,8 +2602,9 @@ void fbcon_suspended(struct fb_info *info)
return;
vc = vc_cons[par->currcon].d;
- /* Clear cursor, restore saved data */
- fbcon_cursor(vc, false);
+ /* Don't need to clear cursor and restore saved data in graphic mode */
+ if (vc->vc_mode != KD_GRAPHICS)
+ fbcon_cursor(vc, false);
I think checking for "== KD_TEXT" is probably better.
And, maybe using con_is_visible(vc).
So:
+ if (con_is_visible(vc) && (vc->vc_mode == KD_TEXT))
+ fbcon_cursor(vc, false);
void fbcon_resumed(struct fb_info *info)
@@ -2615,7 +2616,9 @@ void fbcon_resumed(struct fb_info *info)
return;
vc = vc_cons[par->currcon].d;
- update_screen(vc);
+ /* Graphics mode is managed by userspace */
+ if (vc->vc_mode != KD_GRAPHICS)
here the same check as above... ?
Helge